/* 
 * Desc:
 *
 * Created by Magnus Berglund
 * 
 * Date: 
 */

#define VERSION "1.2.2"

#define error(args...) ({ fprintf(stderr, args); exit(1); })

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <ctype.h>
#include "truesound.h"

void usage(const char *prgname) {
    fprintf(stderr, 
	    "Usage: %s [OPTION...]\n"
	    "Control the TrueSound Decoder board\n"
	    "  -d, --device            choose card (0,1,2) default 0\n"
	    "  -r, --reset             reset the board\n"
	    "  -f, --flush             flush the internal buffers\n"
	    "  -p, --pause             pause the current song\n"
	    "  -c, --cont              continue\n"
	    "  -v, --volume=VOL        set the board volume to 'VOL' (0-100)\n"
	    "  -g, --getvol            get the current volume\n"
	    "  -h, --help              print this help\n"
	    "      --version           print version\n",
	    prgname);
    
}

int main(int argc, char **argv)
{
    int i;
    int devfd;
    int ret;
    int opt;
    unsigned char volume;
    char *progname;
    int device=0;
    char *devnames[] = { "/dev/tsd0-ioctl", 
			 "/dev/tsd1-ioctl", 
			 "/dev/tsd2-ioctl", 
			 NULL };

    int reset = 0;
    int flush = 0;
    int pause = 0;
    int cont = 0;
    int vol = 0;
    int getvol = 0;

    struct option long_options[]=
    {
	{"device",  1, 0, 'd'},
	{"reset",   0, 0, 'r'},
	{"flush",   0, 0, 'f'},
	{"pause",   0, 0, 'p'},
	{"cont",    0, 0, 'c'},
	{"volume",  1, 0, 'v'},
	{"getvol",  0, 0, 'g'},
	{"help",    0, 0, 'h'},
	{"version", 0 ,0, 1},
	{0, 0, 0, 0}
    };
    
    progname=argv[0];

    while ( (opt=getopt_long(argc, argv, "d:rfpcv:gh", long_options, 0))!=-1)
	{
	    switch(opt)
		{
		case 'd':
		    i=0;
		    while(optarg[i]!=0) 
			if (!isdigit(optarg[i++]))
			    error("%s: invalid device: %s\n", 
				  progname, optarg);
		    device = atoi(optarg);
		    if (device<0 || device>2)
			error("%s: device option must be 0,1 or 2\n",
			      progname);
		    break;
		case 'r': reset=1; break;
		case 'f': flush=1; break;
		case 'p': pause=1; break;
		case 'c': cont=1; break;
		case 'v': 
		    vol=1;    i=0;
		    while(optarg[i]!=0) 
			if (!isdigit(optarg[i++]))
			    error("%s: invalid volume: %s\n", 
				  progname, optarg);
		    volume = (unsigned char)atoi(optarg);
		    break;
		case 'g': getvol=1; break;
		case 'h':
		    usage(progname);
		    exit(0);
		case 1:
		    fprintf(stderr, "%s version %s, compiledate %s\n", 
			    progname, VERSION, __DATE__);
		    exit(0);
		default:
		    error("Try `%s --help` for more information\n", 
			  progname);
		}

	}

    if (!reset && !flush && !pause && !cont && !vol && !getvol) {
	usage(progname);
	exit(0);
    }
    
    devfd = open(devnames[device], O_WRONLY);
    if (devfd == -1) {
	perror("open");
	exit(1);
    }

    if (reset) {
	ret = ioctl(devfd, TRUESOUND_RESET);
	if (ret) perror("ioctl");
    }
    if (flush) {
	ret = ioctl(devfd, TRUESOUND_FLUSH_BUFFERS);
	if (ret) perror("ioctl");
    }
    if (pause) {
	ret = ioctl(devfd, TRUESOUND_PAUSE);
	if (ret) perror("ioctl");
    }
    if (cont) {
	ret = ioctl(devfd, TRUESOUND_CONT);
	if (ret) perror("ioctl");
    }
    if (vol) {
	ret = ioctl(devfd, TRUESOUND_SETVOL, &volume);
	if (ret) perror("ioctl");
    }
    if (getvol) {
	ret = ioctl(devfd, TRUESOUND_GETVOL, &volume);
	if (ret) perror("ioctl");
	printf("%d\n", volume);
    }
    
    return 0;

}
